import "Scripts/Functions/TableSaveLoad.lua"

Script.Filename = "" --string
Script.FolderName = "" --string

function Script:GetFilename()--arg
	return self.Filename
end

function Script:SaveData()--in
	local saveData = {}
	local filePath = FileSystem:GetAppDataPath()
	local finalPath = filePath.."\\"..self.FolderName.."\\"..self.Filename

	-- create the directory. if it already exists nothing will happen
	FileSystem:CreateDir(filePath.."\\"..self.FolderName)

	for i = 0, App.world:CountEntities() - 1 do
		local entity = App.world:GetEntity(i)
		local name = entity:GetKeyValue("name", "")
		local prefab = entity:GetKeyValue("prefab", "")
		local save = entity:GetKeyValue("save", "false")

		-- for all entities that need to be saved call it's Save() method
		if string.lower(save) == "true" then
			if entity.script ~= nil then
				if entity.script.SaveData ~= nil then

					-- this should return a table of values to save to file
					local data = entity.script:SaveData()

					-- if this was a prefab then create a Prefab property with the prefab path so the load process can load it
					if prefab ~= "" then
						data.Prefab = prefab
					end

					-- add the save data to the final table using this entities ID/Name as the key
					saveData[name] = data
				end
			end
		end
	end

	-- save table to file
	local err = table.save(saveData, finalPath)

	-- write out error if any exists
	if err ~= nil then
		System:Print("Error in saving table: "..err)
	end
end

function Script:LoadData(filename)--in
	local filePath = FileSystem:GetAppDataPath()
	local finalPath = filePath.."\\"..self.FolderName.."\\"..self.Filename
	local data, err = table.load(finalPath)
	
	-- write out error if any exists and return
	if err ~= nil then
		System:Print("Error in reading table: "..err)
		return
	end

	-- loop over all the entity tables in the saved file table and look for the entities in the map
	-- if they don't exist it means they were dynamically loaded so load the prefab for it and then call it's LoadData()
	for k, v in pairs(data) do
		local entityName = k

		-- see if this entity was created in design time
		local entity = self:FindEntityByName(entityName)

		-- was created dynamically and not in the map at design time if entity was null
		if entity == nil then
			entity = Prefab:Load(v.Prefab)

			-- be sure to change it's name to what it is in the saved file
			entity:SetKeyValue("name", entityName)
		end

		-- call the entities load function
		if entity ~= nil then
			if entity.script ~= nil then
				if entity.script.LoadData ~= nil then
					entity.script:LoadData(v)
				end
			end			
		end
	end
end

function Script:FindEntityByName(name)
	for i = 0, App.world:CountEntities() - 1 do
		local entity = App.world:GetEntity(i)
		local entityName = entity:GetKeyValue("name", "")

		if name == entityName then
			return entity
		end
	end

	return nil
end